# This file is part of youtube-links.
#
# Copyright 2025 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License version 3, as published by the Free
# Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranties of MERCHANTABILITY, SATISFACTORY
# QUALITY, or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
#
# You should have received a copy of the GNU Lesser General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

"""Simple integration tests for youtube-links extension."""

# Ignore import organization warnings
# ruff: noqa: E402
# ruff: noqa: PLC0415

import sys
from importlib import import_module
from pathlib import Path
from typing import cast

from docutils import nodes
from docutils.statemachine import StringList

# Add the extension to the path
sys.path.insert(0, str(Path(__file__).parents[2] / "sphinx_youtube_links"))


def test_extension_can_be_imported():
    """Test that the extension can be imported without errors."""
    try:
        sphinx_youtube_links = import_module("sphinx_youtube_links")

        assert hasattr(sphinx_youtube_links, "setup")
        assert callable(sphinx_youtube_links.setup)
        assert hasattr(sphinx_youtube_links, "YouTubeLink")
    except ImportError as e:
        pytest.fail(f"Failed to import sphinx_youtube_links: {e}")


def test_extension_setup_function():
    """Test that the setup function returns correct metadata."""
    from unittest.mock import Mock

    import sphinx_youtube_links

    app_mock = Mock()
    app_mock.add_directive = Mock()

    with patch("sphinx_youtube_links.common.add_css"):
        result = sphinx_youtube_links.setup(app_mock)

    assert "version" in result
    assert "parallel_read_safe" in result
    assert "parallel_write_safe" in result
    assert result["parallel_read_safe"] is True
    assert result["parallel_write_safe"] is True

    # Check that directive was registered
    app_mock.add_directive.assert_called_once_with(
        "youtube", sphinx_youtube_links.YouTubeLink
    )


def test_youtube_directive_instantiation():
    """Test that YouTube directive can be instantiated."""
    from unittest.mock import Mock

    import sphinx_youtube_links

    # Test directive can be created
    directive = sphinx_youtube_links.YouTubeLink(
        name="youtube",
        arguments=["https://www.youtube.com/watch?v=test"],
        options={"title": "Test Title"},
        content=StringList(),
        lineno=1,
        content_offset=0,
        block_text="",
        state=Mock(),
        state_machine=Mock(),
    )

    assert directive.required_arguments == 1
    assert directive.optional_arguments == 0
    assert directive.has_content is False
    directive_spec = directive.option_spec if directive.option_spec else {}
    assert "title" in directive_spec


def test_youtube_directive_execution():
    """Test that YouTube directive can be executed."""
    from unittest.mock import Mock

    import sphinx_youtube_links

    directive = sphinx_youtube_links.YouTubeLink(
        name="youtube",
        arguments=["https://www.youtube.com/watch?v=test"],
        options={"title": "Test Title"},
        content=StringList(),
        lineno=1,
        content_offset=0,
        block_text="",
        state=Mock(),
        state_machine=Mock(),
    )

    result = directive.run()

    assert len(result) == 1
    raw_node = cast(nodes.raw, result[0])
    assert raw_node.tagname == "raw"
    assert raw_node.children

    html_content = str(raw_node.children[0])
    assert "Test Title" in html_content
    assert "https://www.youtube.com/watch?v=test" in html_content
    assert "youtube_link" in html_content


# Import necessary modules
import shutil
import subprocess
from unittest.mock import patch

import bs4
import pytest


@pytest.fixture
def example_project(request) -> Path:
    project_root = request.config.rootpath
    example_dir = project_root / "tests/integration/example"

    target_dir = Path().resolve() / "example"
    shutil.copytree(example_dir, target_dir, dirs_exist_ok=True)

    return target_dir


@pytest.mark.slow
def test_sphinx_build(example_project):
    """Ensure that Sphinx builds successfully."""
    build_dir = example_project / "_build"
    subprocess.check_call(
        ["sphinx-build", "-b", "html", "-W", example_project, build_dir],
    )

    index = build_dir / "index.html"

    # Rename the test output to something more meaningful
    shutil.copytree(
        build_dir, build_dir.parents[1] / ".test_output", dirs_exist_ok=True
    )
    soup = bs4.BeautifulSoup(index.read_text(), features="lxml")

    shutil.rmtree(example_project)  # Delete copied source

    ext_text = soup.find("p")
    if ext_text:
        print("Successful build.")
    else:
        pytest.fail("Directive output not found in document.")



"""Defines the youtube directive and connects it to Sphinx."""

# This file is part of youtube-links.
#
# Copyright 2025 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License version 3, as published by the Free
# Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranties of MERCHANTABILITY, SATISFACTORY
# QUALITY, or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
#
# You should have received a copy of the GNU Lesser General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

from docutils import nodes
from docutils.parsers.rst import Directive
from docutils.parsers.rst import directives
from sphinx_youtube_links import common
import requests
from sphinx.application import Sphinx
from sphinx.util.typing import ExtensionMetadata
from bs4 import BeautifulSoup

cache: dict[str, str] = {}


class YouTubeLink(Directive):
    """Define the youtube directive's arguments and behavior."""

    required_arguments = 1
    optional_arguments = 0
    has_content = False
    option_spec = {"title": directives.unchanged}

    def run(self) -> list[nodes.Node]:
        """Generate a raw HTML node containing the formatted link."""
        title: str = ""

        if "title" in self.options:
            title = self.options["title"]
        elif self.arguments[0] in cache:
            title = cache[self.arguments[0]]
        else:
            try:
                r = requests.get(self.arguments[0], timeout=10)
                r.raise_for_status()
                soup = BeautifulSoup(r.text, "html.parser")
                if soup.title:
                    title = soup.title.get_text()
                    cache[self.arguments[0]] = title
            except requests.HTTPError as err:
                print(err)

        fragment: str = f"""
            <p class="youtube_link">
              <a href="{self.arguments[0]}" target="_blank">
                <span title="{title}" class="play_icon">▶</span>
                <span title="{title}">Watch on YouTube</span>
              </a>
            </p>
        """
        raw = nodes.raw(text=fragment, format="html")

        return [raw]


def setup(app: Sphinx) -> ExtensionMetadata:
    """Connect the extension and its assets to Sphinx.

    app (Sphinx): Sphinx application instance.
    """
    app.add_directive("youtube", YouTubeLink)
    common.add_css(app, "youtube.css")

    return {"version": "0.1", "parallel_read_safe": True, "parallel_write_safe": True}



# This file is part of youtube-links.
#
# Copyright 2025 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License version 3, as published by the Free
# Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranties of MERCHANTABILITY, SATISFACTORY
# QUALITY, or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
#
# You should have received a copy of the GNU Lesser General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

"""Unit tests for youtube-links extension."""

# Ignore import organization warnings
# ruff: noqa: E402
# ruff: noqa: PLC0415

from unittest.mock import Mock, patch

from docutils.parsers.rst import directives
from docutils.statemachine import StringList
from sphinx.application import Sphinx
from sphinx_youtube_links import YouTubeLink, setup


class TestYouTubeLinksSetup:
    """Test the extension setup function."""

    def test_setup_returns_metadata(self):
        """Test that setup returns proper extension metadata."""
        app_mock = Mock(spec=Sphinx)
        app_mock.add_directive = Mock()

        with patch("sphinx_youtube_links.common.add_css") as mock_add_css:
            result = setup(app_mock)

        assert result.get("version", "") == "0.1"
        assert result.get("parallel_read_safe", "") is True
        assert result.get("parallel_write_safe", "") is True

        # Verify the directive is registered
        app_mock.add_directive.assert_called_once_with("youtube", YouTubeLink)

        # Verify CSS is added
        mock_add_css.assert_called_once_with(app_mock, "youtube.css")


class TestYouTubeLinkDirective:
    """Test the YouTubeLink directive."""

    def setup_method(self):
        """Set up test fixtures."""
        self.directive = YouTubeLink(
            name="youtube",
            arguments=["https://www.youtube.com/watch?v=dQw4w9WgXcQ"],
            options={},
            content=StringList(),
            lineno=1,
            content_offset=0,
            block_text="",
            state=Mock(),
            state_machine=Mock(),
        )

    def test_directive_options(self):
        """Test that the directive has correct options configured."""
        assert YouTubeLink.required_arguments == 1
        assert YouTubeLink.optional_arguments == 0
        assert YouTubeLink.has_content is False

        directive_spec = YouTubeLink.option_spec if YouTubeLink.option_spec else {}
        assert "title" in directive_spec
        assert directive_spec["title"] == directives.unchanged

    def test_run_with_custom_title(self):
        """Test directive execution with custom title."""
        self.directive.options = {"title": "Custom Video Title"}

        result = self.directive.run()

        assert len(result) == 1
        raw_node = result[0]
        # For raw nodes, the HTML content is stored in the node's children as text
        html_content = str(raw_node.children[0]) if raw_node.children else ""
        assert "Custom Video Title" in html_content
        assert "https://www.youtube.com/watch?v=dQw4w9WgXcQ" in html_content
        assert "youtube_link" in html_content
        assert "Watch on YouTube" in html_content

    @patch("sphinx_youtube_links.requests.get")
    @patch("sphinx_youtube_links.BeautifulSoup")
    def test_run_with_automatic_title(self, mock_bs, mock_get):
        """Test directive execution with automatic title extraction."""
        # Mock successful HTTP response
        mock_response = Mock()
        mock_response.raise_for_status.return_value = None
        mock_response.text = (
            "<html><head><title>Amazing Video - YouTube</title></head></html>"
        )
        mock_get.return_value = mock_response

        # Mock BeautifulSoup
        mock_soup = Mock()
        mock_soup.title.get_text.return_value = "Amazing Video - YouTube"
        mock_bs.return_value = mock_soup

        # Clear the cache to ensure fresh request
        sphinx_youtube_links.cache.clear()

        result = self.directive.run()

        assert len(result) == 1
        raw_node = result[0]
        html_content = str(raw_node.children[0]) if raw_node.children else ""
        assert "Amazing Video - YouTube" in html_content
        assert "https://www.youtube.com/watch?v=dQw4w9WgXcQ" in html_content

        mock_get.assert_called_once_with(
            "https://www.youtube.com/watch?v=dQw4w9WgXcQ", timeout=10
        )

    @patch("sphinx_youtube_links.requests.get")
    def test_run_with_http_error(self, mock_get):
        """Test directive execution when HTTP request fails."""
        import requests

        # Mock HTTP error
        mock_get.side_effect = requests.HTTPError("404 Not Found")

        # Clear the cache to ensure fresh request
        sphinx_youtube_links.cache.clear()

        with patch("builtins.print") as mock_print:
            result = self.directive.run()

        # Should still return a result, but without title from HTTP
        assert len(result) == 1
        raw_node = result[0]
        html_content = str(raw_node.children[0]) if raw_node.children else ""
        assert "https://www.youtube.com/watch?v=dQw4w9WgXcQ" in html_content

        # Should have printed the error
        mock_print.assert_called_once()

    def test_run_uses_cache(self):
        """Test that the directive uses cached titles."""
        # Pre-populate cache
        url = "https://www.youtube.com/watch?v=dQw4w9WgXcQ"
        sphinx_youtube_links.cache[url] = "Cached Title"

        result = self.directive.run()

        assert len(result) == 1
        raw_node = result[0]
        html_content = str(raw_node.children[0]) if raw_node.children else ""
        assert "Cached Title" in html_content

    def test_html_output_structure(self):
        """Test that the generated HTML has the correct structure."""
        self.directive.options = {"title": "Test Title"}

        result = self.directive.run()
        raw_node = result[0]
        html_content = str(raw_node.children[0]) if raw_node.children else ""

        # Check for expected HTML structure
        assert 'class="youtube_link"' in html_content
        assert 'target="_blank"' in html_content
        assert 'class="play_icon"' in html_content
        assert "▶" in html_content  # Play icon
        assert "Watch on YouTube" in html_content


# Import the module after defining the tests
import sphinx_youtube_links
